/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package nl.cloudfarming.client.sensor.area;

import java.awt.Point;
import java.awt.image.DataBuffer;
import java.awt.image.WritableRaster;
import java.util.List;
import nl.cloudfarming.client.model.SensorData;
import org.geotools.coverage.grid.GridCoverage2D;
import org.geotools.coverage.grid.GridCoverageFactory;
import org.geotools.geometry.jts.ReferencedEnvelope;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.opengis.geometry.BoundingBox;

/**
 *
 * @author Timon Veenstra
 */
public class RasterFactory {

    /**
     * create a raster from sensor data
     *
     *
     * @param sensorData
     * @return raster
     */
    public static GridCoverage2D createRaster(List<SensorData> sensorData){

        if (sensorData.isEmpty()){
            return null;
        }
        double minx = sensorData.get(0).getLongtitude();
        double maxx = sensorData.get(0).getLongtitude();
        double miny = sensorData.get(0).getLatitude();
        double maxy = sensorData.get(0).getLatitude();

        for (SensorData sd : sensorData) {
            minx = (minx < sd.getLongtitude()) ? minx : sd.getLongtitude();
            miny = (miny < sd.getLatitude()) ? miny : sd.getLatitude();
            maxx = (maxx > sd.getLongtitude()) ? maxx : sd.getLongtitude();
            maxy = (maxy > sd.getLatitude()) ? maxy : sd.getLatitude();
        }

        // width height and ration in coordiantes
        double diffx = maxx - minx;
        double diffy = maxy - miny;
        double ratio = diffy / diffx;


        // in pixels
        Double width = Math.ceil(Math.sqrt(sensorData.size() / ratio));
        Double height = Math.ceil(Math.sqrt(sensorData.size() * ratio));

        int w = width.intValue();
        int h = height.intValue();
        int bands = 1;
        Point location = new Point(0, 0);
        WritableRaster raster = WritableRaster.createBandedRaster(DataBuffer.TYPE_INT, w, h, bands, location);


        for (SensorData sd : sensorData) {
            Double x = Math.floor((sd.getLongtitude() - minx) / (diffx / (width-1)));
            Double y = Math.floor((sd.getLatitude() - miny) / (diffy / (height-1)));

            //TODO sample data gets casted into an int, so multuiply with 1000 for example
            raster.setSample(x.intValue(), y.intValue(), 0, sd.getReading1());
//            raster.setSample(x.intValue(), y.intValue(), 1, sd.getReading2());
//            raster.setSample(x.intValue(), y.intValue(), 2, sd.getReading3());
//            raster.setSample(x.intValue(), y.intValue(), 3, sd.getCalculatedIndex1());
//            raster.setSample(x.intValue(), y.intValue(), 4, sd.getCalculatedIndex2());
        }

        BoundingBox boundingBox = new ReferencedEnvelope(minx,maxx,miny,maxy, DefaultGeographicCRS.WGS84);


        GridCoverageFactory factory = new GridCoverageFactory();
        GridCoverage2D coverage2D = factory.create("name", raster, boundingBox);

        return coverage2D;
    }
}
